

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'EntityFactory.dart';
import 'base_entity.dart';
import 'ErrorEntity.dart';
import 'http_config.dart';
import 'http_method.dart';
import 'json_convert_help.dart';

class DioManager{
  static final DioManager _shared = DioManager._internal();
  factory DioManager() => _shared;

  Dio dio;

  DioManager._internal() {
    if (null == dio) {
      BaseOptions options = BaseOptions(
          contentType: Headers.jsonContentType,
          responseType: ResponseType.json,
          receiveDataWhenStatusError: false,
          connectTimeout: HttpConfig.ConnectTimeout,
          receiveTimeout: HttpConfig.ReceiveTimeout
      );
      dio = Dio(options);

    }

  }

  /**
   * 添加拦截器
   */
  DioManager addIntercept(Interceptor interceptor) {
    dio.interceptors.add(interceptor);
    return this;
  }

  /**
   * 设置BaseUrl
   */
  DioManager setBaseUrl(String baseUrl) {
    dio.options.baseUrl = baseUrl;
    return this;
  }

  /**
   * 设置连接超时
   */
  DioManager setConnectTimout(int connectTimeout) {
    dio.options.connectTimeout = connectTimeout;
    return this;
  }

  /**
   * 设置响应超时
   */
  DioManager setReceiveTimeout(int receiveTimeout) {
    dio.options.receiveTimeout = receiveTimeout;
    return this;
  }

  /**
   * 设置Json转换的方法
   */
  setJsonConvertHelp(JsonConvertHelp help) {
    EntityFactory.setJsonConvertHelp(help);
  }

  Future request<T>({
    @required HTTPMethod method,
    @required String url,
    Map params,
    String baseUrl,
    @required Function(T) success,
    @required Function(ErrorEntity) error
  }) async {
    try {
      if (null != baseUrl && !baseUrl.isEmpty) {
        dio.options.baseUrl = baseUrl;
      }
      print("start request");
      Response response = await dio.request(
                  url, queryParameters: params,
                  options: Options(method: HTTPMethodValues[method]),
                  );
      print("response: " + response.toString());
      if (null != response) {
        BaseEntity<T> entity = BaseEntity<T>.fromJson(response.data);
//        BaseEntity entity = response.data;
        print("entity.code: ${entity.code}, entity.message: ${entity.message}, entity.data: ${entity.data}");
        if (entity.code == BaseEntity.STATUS_OK) {
          success(entity.data);
        } else {
          error(ErrorEntity(code: entity.code, message: entity.message));
        }
      } else {
        error(ErrorEntity(code: -1, message: "未知错误！"));
      }
    } on DioError catch(e) {
      error(createErrorEntity(e));
    }
  }

  ErrorEntity createErrorEntity(DioError e) {
    switch (e.type) {
      case DioErrorType.CANCEL:
        return ErrorEntity(code: -1, message: "请求取消！");
      case DioErrorType.CONNECT_TIMEOUT:
        return ErrorEntity(code: -1, message: "连接超时!");
      case DioErrorType.SEND_TIMEOUT:
        return ErrorEntity(code: -1, message: "发送超时！");
      case DioErrorType.RECEIVE_TIMEOUT:
        return ErrorEntity(code: -1, message: "响应超时！");
      case DioErrorType.RESPONSE:
        try {
          int errCode = e.response.statusCode;
          String errMsg = e.response.statusMessage;
          return ErrorEntity(code: errCode, message: errMsg);
        } on Exception catch(exception) {
          return ErrorEntity(code: -1, message: "未知错误！");
        }
        break;
      default:
        return ErrorEntity(code: -1, message: e.message);
    }
  }
}